////////////////////////////////////////////////////////////////////////////////
/// @file     usb_desc.C
/// @author   AE TEAM
/// @brief    Descriptors for Custom HID Demo.
////////////////////////////////////////////////////////////////////////////////
/// @attention
///
/// THE EXISTING FIRMWARE IS ONLY FOR REFERENCE, WHICH IS DESIGNED TO PROVIDE
/// CUSTOMERS WITH CODING INFORMATION ABOUT THEIR PRODUCTS SO THEY CAN SAVE
/// TIME. THEREFORE, MINDMOTION SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT OR
/// CONSEQUENTIAL DAMAGES ABOUT ANY CLAIMS ARISING OUT OF THE CONTENT OF SUCH
/// HARDWARE AND/OR THE USE OF THE CODING INFORMATION CONTAINED HEREIN IN
/// CONNECTION WITH PRODUCTS MADE BY CUSTOMERS.
///
/// <H2><CENTER>&COPY; COPYRIGHT MINDMOTION </CENTER></H2>
////////////////////////////////////////////////////////////////////////////////

// Define to prevent recursive inclusion
#define _USB_DESC_C_

// Files includes
#include "usb_desc.h"

////////////////////////////////////////////////////////////////////////////////
/// @addtogroup MM32_Hardware_Abstract_Layer
/// @{

////////////////////////////////////////////////////////////////////////////////
/// @addtogroup USB_DESC
/// @{

////////////////////////////////////////////////////////////////////////////////
/// @addtogroup USB_Exported_Functions
/// @{

// Private typedef -----------------------------------------------------------
// Private define ------------------------------------------------------------
// Private macro -------------------------------------------------------------
// Private variables ---------------------------------------------------------
// Extern variables ----------------------------------------------------------
// Private function prototypes -----------------------------------------------
// Private functions ---------------------------------------------------------

// USB Standard Device Descriptor
uint8 MIDI_DeviceDescriptor[MIDI_SIZ_DEVICE_DESC] = {
  MIDI_SIZ_DEVICE_DESC,          // bLength: 18 
  USB_DEVICE_DESCRIPTOR_TYPE,    // descriptor type 

  0x00,                    // bcdUSB LSB: USB release number -> USB2.0 
  0x02,                    // bcdUSB MSB: USB release number -> USB2.0 

  0xEF,                    // bDeviceClass:      
  0x02,                    // bDeviceSubClass: 
  0x01,                    // bDeviceProtocol: 
  0x40,        // bMaxPacketSize0:  LowS(8), FullS(8,16,32,64), HighS(64)  

  LOWORD(VENDOR_ID),       // idVendor LSB:  
  HIWORD(VENDOR_ID),       // idVendor MSB:  

  LOWORD(PRODUCT_ID),      // idProduct LSB:  
  HIWORD(PRODUCT_ID),      // idProduct MSB:  

  LOWORD(DEVICE_VERSION),  // bcdDevice LSB:  
  HIWORD(DEVICE_VERSION),  // bcdDevice MSB:  

  DEV_COMPANY_STR_IDX,     // iManufacturer: Index of string descriptor describing manufacturer 
  DEV_PROD_STR_IDX,        // iProduct: Index of string descriptor describing product 
  DEV_SN_STR_IDX,          // iSerialNumber: not use
  0x01                     // bNumConfigurations: number of configurations 
}
; // MIDI_DeviceDescriptor


// USB Configuration Descriptor
//   All Descriptors (Configuration, Interface, Endpoint, Class, Vendor
uint8 MIDI_ConfigDescriptor[MIDI_SIZ_CONFIG_DESC] = {

	/************** Descriptor of standard audio +HID Configuration ****************///???????
    0x09,         /* bLength: Configuration Descriptor size */
    USB_CONFIGURATION_DESCRIPTOR_TYPE, /* bDescriptorType: Configuration */
    MIDI_SIZ_CONFIG_DESC,         /* wTotalLength: Bytes returned */
    0x00,
    0x03,         /* bNumInterfaces: 1 HID interface +MIDI 2 interface 03*/
    0x01,         /* bConfigurationValue: Configuration value */
    0x00,         /* iConfiguration: Index of string descriptor describingthe configuration*/
    0x00,         /* bmAttributes: Self powered */
    0x32,         /* MaxPower 0x32:100 mA */

/************** Descriptor of ̨??? HID ????? ****************/	  
    /* Interface Descriptor 0/0 HID, 2 Endpoints */
    0x09,         /* bLength: Interface Descriptor size */
    USB_INTERFACE_DESCRIPTOR_TYPE,/* bDescriptorType: Interface descriptor type */
    0x00,         /* bInterfaceNumber: Number of Interface */                                  
    0x00,         /* bAlternateSetting: Alternate setting */  
    0x02,         /* bNumEndpoints */                                   
    0x03,         /* bInterfaceClass: HID */
    0x00,         /* bInterfaceSubClass : 1=BOOT, 0=no boot */
    0x00,         /* nInterfaceProtocol : 0=none, 1=keyboard, 2=mouse */           
    0x00,            /* iInterface: Index of string descriptor */            
    /******************** Descriptor of  HID ??? ********************/
    0x09,         /* bLength: HID Descriptor size */
    HID_DESCRIPTOR_TYPE, /* bDescriptorType: HID *///0x21
    0x11,         /* bcdHID: HID Class Spec release number */
    0x01,
    0x00,         /* bCountryCode: Hardware target country */
    0x01,         /* bNumDescriptors: Number of HID class descriptors to follow */
    0x22,         /* bDescriptorType */ 
    /* wItemLength: Total length of Report descriptor */
    HID_SIZ_REPORT_DESC,              //??????䨮D?
    0x00,
	
     /******************** Descriptor of Custom HID endpoints ??????******************/
    /* Endpoint Descriptor 81 1 In, Interrupt, 1 ms */       
    0x07,          /* bLength: Endpoint Descriptor size */
    USB_ENDPOINT_DESCRIPTOR_TYPE, /* bDescriptorType: */
    /*	Endpoint descriptor type */
    0x82,          /* bEndpointAddress: Endpoint Address (IN) */            
    0x03,          /* bmAttributes: Interrupt endpoint */                       
    0x20,          /* wMaxPacketSize: 64 Bytes max */                      
    0x00,
    0x01,          /* bInterval: Polling Interval (1 ms) */                
    /* Endpoint Descriptor 02 2 Out, Interrupt, 1 ms*/ 	  	

    0x07,	/* bLength: Endpoint Descriptor size */
    USB_ENDPOINT_DESCRIPTOR_TYPE,	/* bDescriptorType: */
	/*	Endpoint descriptor type */
    0x02,	/* bEndpointAddress: */ /*	Endpoint Address (OUT) */                             
    0x03,	/* bmAttributes: Interrupt endpoint */
    0x20,	/* wMaxPacketSize: 32 Bytes max */                      
    0x00,
    0x01,          /* bInterval: Polling Interval (1 ms) */                 
 
   /* MIDI Interface Descriptor */
   0x09,                              /* bLength */
   USB_INTERFACE_DESCRIPTOR_TYPE,     /* bDescriptorType */
   0x01,       						  /* bInterfaceNumber */
   0x00,                              /* bAlternateSetting */
   0x00,                              /* bNumEndpoints */
   0x01,                              /* bInterfaceClass */
   0x01,                              /* bInterfaceSubClass */
   0x00,                              /* bInterfaceProtocol */
   0x00,                              /* iInterface */

   /* AudioControl Interface Descriptor */
   0x09,                              /* bLength */
   0x24,     /* bDescriptorType */
   0x01,                              /* HEADER subtype */               
   0x00,							  /* Revision of class specification - 01.00 */
   0x01,                   
   0x09,							  /* Total size of class specific descriptors - 0009 */
   0x00,
   0x01,                              /* Number of streaming interfaces */
   0x02,                              /* MIDIStreaming interface -> AudioControl interface */

   /* Interface Descriptor */
   0x09,  /* bLength */
   USB_INTERFACE_DESCRIPTOR_TYPE,     /* bDescriptorType */
   0x02,                              /* bInterfaceNumber */
   0x00,                              /* bAlternateSetting */
   0x02,                              /* bNumEndpoints : 2 */
   0x01,                              /* Audio Class */
   0x03,                              /* MIDI Streaming */
   0x00,                              /* bInterfaceProtocol */
   0x00,                              /* iInterface */

   /* MIDIStreaming Interface Descriptor */
   0x07,              // bFunction Length
   0x24, 
   0x01,             // bDescriptor subtype: HEADER
   0x00,			 // iADC - 01.00
   0x01,
   0x41,			 // iLength	- 0x0041
   0x00,

   /* MIDIStreaming Interface Descriptor */
   0x06, // bFunctionLength
   0x24, 
   0x02, // bDescriptor Subtype: MIDI_IN_JACK
   0x01, // bJackType: Embeded
   0x01, // bJackID: 1
   0x00, //DEV_MIDI_JACK1_IDX, // bDescriptor

   /* MIDIStreaming Interface Descriptor */
   0x06, // bFunctionLength
   0x24, 
   0x02, // bDescriptor Subtype: MIDI_IN_JACK
   0x02, // bJackType: External
   0x02, // bJackID: 2
   0x00, // bDescriptor

   /* MIDIStreaming Interface Descriptor */
   0x09, // bFunctionLength
   0x24, 
   0x03, // bDescriptor Subtype: MIDI_OUT_JACK
   0x01, // bJackType: Embeded
   0x03, // bJackID: 3
   0x01, // bNrInputPins : 1
   0x02, // bSourceID : 2
   0x01, // bSourcePin : 1
   0x00, //DEV_MIDI_JACK3_IDX, // bJack : 0

   /* MIDIStreaming Interface Descriptor */
   0x09, // bFunctionLength
   0x24, 
   0x03, // bDescriptor Subtype: MIDI_OUT_JACK
   0x02, // bJackType: External
   0x04, // bJackID: 4
   0x01, // bNrInputPins : 1
   0x01, // bSourceID : 2
   0x01, // bSourcePin : 1
   0x00, // bJack : 0
	
   /* Endpoint2 Descriptor */
   0x09,   // bLength	   
   USB_ENDPOINT_DESCRIPTOR_TYPE, 
   0x01,   // bEndpointAddress, Endpoint 01- OUT
   0x02,   // bmAttributes      BULK
   0x40,   // wMaxPacketSize :  0x0040
   0x00,
   0x00,   // bInterval
   0x00,   // bRefresh
   0x00,   // bSynchAddress

   /* ˵1 */
   0x05,   // bLength
   0x25,   
   0x01,   // bDescriptorSubtype
   0x01,   // bNumEmbMIDIJack
   0x01,    // bAssoctackID-Embeded MIDIIN

   /* Endpoint3 Descriptor */
   0x09,   // bLength
   USB_ENDPOINT_DESCRIPTOR_TYPE,   
   0x81,   // bEndpointAddress, Endpoint 81 - IN
   0x02,   // bmAttributes      BULK
   0x40,   // wMaxPacketSize :  0x0040
   0x00,
   0x00,   // bInterval
   0x00,   // bRefresh
   0x00,   // bSynchAddress

   /* Endpoint1 Descriptor */
   0x05,   // bLength
   0x25,   
   0x01,   // bDescriptorSubtype
   0x01,   // bNumEmbMIDIJack
   0x03    // bAssoctackID-Embeded MIDIOUT
}
; // MIDI_ConfigDescriptor
uint8 HidMIDI_ReportDescriptor[HID_SIZ_REPORT_DESC] = {
  0x06, 0x01, 0xff,  // USAGE_PAGE (Vendor HID Page)
  0x09, 0x00,        // USAGE (Undefined)
  0xa1, 0x01,        // COLLECTION (Application)

  // Controller Report 0x60
  0x09, 0x01,        //  USAGE (FW Data Report)
  0xa1, 0x02,        //  COLLECTION (Logical)
  0x85, 0x60,        //  0000: REPORT_ID (0x60)

  0x09, 0x61,        //   USAGE (FW Data)
	                   //   0001: Function ID
                     //   0002: Function Informaion
                     //   0003: Data Length LSB
                     //   0004: Data Length MSB
                     //   0005~0062: Data 1-57
                     //   0063: CRC 	
  0x15, 0x00,        //   LOGICAL_MINIMUM (0)
  0x26, 0xFF, 0x00,  //   LOGICAL_MAXIMUM (255)
  0x75, 0x08,        //   REPORT_SIZE     (8)
  0x95, 0x3F,        //   REPORT_COUNT    (63)
  0xb1, 0x02,        //   FEATURE (Data,Var,Abs)

  0xc0,              //  END_COLLECTION	

  // FW Data Report 0x70
  0x09, 0x01,        //  USAGE (FW Data Report)
  0xa1, 0x02,        //  COLLECTION (Logical)
  0x85, 0x70,        //  0000: REPORT_ID (0x70)

  0x09, 0x71,        //   USAGE (FW Data)
	                   //   0001: Function ID
                     //   0002: Function Informaion
                     //   0003: Data Length LSB
                     //   0004: Data Length MSB
										 //   0005: Chip ID
                     //   0006: Page LSB				
                     //   0007: Page MSB
                     //   0008~0014: Reserve									 
                     //   0015: FW Data 1
	                   //   1038: FW Data 1024 
                     //   1039: CRC 	
  0x15, 0x00,        //   LOGICAL_MINIMUM (0)
  0x26, 0xFF, 0x00,  //   LOGICAL_MAXIMUM (255)
  0x75, 0x08,        //   REPORT_SIZE     (8)
  0x96, 0x3F, 0x04,  //   REPORT_COUNT    (1087)
  0xb1, 0x02,        //   FEATURE (Data,Var,Abs)

  0xc0,              //  END_COLLECTION	

  0xc0,              // END_COLLECTION

   
}; // MIDI_ReportDescriptor

// USB String Descriptors (optional)
 uint8 MIDI_StringLangID[MIDI_SIZ_STRING_LANGID] = {
    MIDI_SIZ_STRING_LANGID,
    USB_STRING_DESCRIPTOR_TYPE,
    0x09,
    0x04  
}
; // LangID = 0x0409: U.S. English

//YiQi
uint8 MIDI_StringVendor[MIDI_SIZ_STRING_VENDOR] = {
    MIDI_SIZ_STRING_VENDOR, // Size of Vendor string
    USB_STRING_DESCRIPTOR_TYPE,  // bDescriptorType
    // Manufacturer: " "
    'M', 0,
	'i', 0, 
	'D', 0,
	'i', 0, 
};

//Artplay X1 Pro
uint8 MIDI_StringProduct[MIDI_SIZ_STRING_PRODUCT] = {
    MIDI_SIZ_STRING_PRODUCT,          // bLength
    USB_STRING_DESCRIPTOR_TYPE,        // bDescriptorType
    'A', 0,
	'r', 0, 
	't', 0,
	'P', 0, 
	'l', 0,
	'a', 0, 
	'y', 0,
    ' ', 0, 
	'X', 0, 
	'1', 0, 
	' ', 0,
	'P', 0, 
	'r', 0, 
	'o', 0,
};

uint8 MIDI_StringHidDfu[MIDI_SIZ_STRING_HIDDFU] =
{
    MIDI_SIZ_STRING_HIDDFU,          // bLength
    USB_STRING_DESCRIPTOR_TYPE,        // bDescriptorType
    'm', 0,
	'i', 0, 
	'd', 0,
	'i', 0, 
	'p', 0,
	'l', 0, 
	'u', 0,
    's', 0, 
	' ', 0, 
	'h', 0, 
	'i', 0,
	'd', 0, 
};

//豸к	
uint8 MIDI_StringSerial[MIDI_SIZ_STRING_SERIAL] =
{
    MIDI_SIZ_STRING_SERIAL,           // bLength 
    USB_STRING_DESCRIPTOR_TYPE,       // bDescriptorType
	  // String: "FFFF FFFF",len=18B    //Zhixing 180504 Change;
    'F',0, 				 
    'F',0, 
    'F',0, 
    'F',0,
    'F',0, 
    'F',0, 
    'F',0,
    'F',0,
	'F',0, 				 
    'F',0, 
    'F',0, 
    'F',0,
    'F',0, 
    'F',0, 
};

/// @}

/// @}

/// @}

